#include <iostream>
#include "dffviewer.h"
using namespace std;

GLubyte carcol1[] = { 234, 205, 81, 255 };
GLubyte carcol2[] = { 80, 40, 200, 255 };

void CGlDff::TransformFrame(RwFrame *f, int which)
{
	GLfloat transmat[16];

	if (f[which].Parent != -1)
		TransformFrame(f, f[which].Parent);

	transmat[0] = f[which].Rotation[0][0];
	transmat[1] = f[which].Rotation[0][1];
	transmat[2] = f[which].Rotation[0][2];
	transmat[3] = 0;

	transmat[4] = f[which].Rotation[1][0];
	transmat[5] = f[which].Rotation[1][1];
	transmat[6] = f[which].Rotation[1][2];
	transmat[7] = 0;

	transmat[8] = f[which].Rotation[2][0];
	transmat[9] = f[which].Rotation[2][1];
	transmat[10] = f[which].Rotation[2][2];
	transmat[11] = 0;

	transmat[12] = f[which].Position[0];
	transmat[13] = f[which].Position[1];
	transmat[14] = f[which].Position[2];
	transmat[15] = 1;

	glMultMatrixf(transmat);
}

void CGlDff::DrawGeometry(RpGeometry &g, CGlTxd &txd, int drawtransp)
{
	RwUInt32 i, j, k, m;
	int isTransparent;
	RpMesh *s;
	RwBool hasAlpha;
	Vector3f *vert, *norm;
	Vector2f *uv;

	avcolor[3] = 255;
	for (j = 0; j < g.numSplits; j++) {
		s = &g.Split[j];
		vert = g.Vertex;
		norm = g.Normal;
		uv = g.UV;

		isTransparent = 0;
		if (UseTextures) {
			glActiveTexture(GL_TEXTURE1);
			glBindTexture(GL_TEXTURE_2D, 0);
			glActiveTexture(GL_TEXTURE0);
			glBindTexture(GL_TEXTURE_2D, 0);

			/* if there is a diffuse map, use it */
			if (g.Mat[s->MatIndex].HasTex) {
				hasAlpha = txd.BindTex(&g.Mat[s->MatIndex].Tex);
				if (hasAlpha)
					isTransparent = 1;
			}

			/* if there is a reflection map, use it */
			if (g.Mat[s->MatIndex].HasRef) {
				glActiveTexture(GL_TEXTURE1);
				txd.BindTex(&g.Mat[s->MatIndex].Reflection);
				glActiveTexture(GL_TEXTURE0);
			}
		}

		if (g.Mat[s->MatIndex].Color[3] != 255)
			isTransparent = 1;

		if (drawtransp != isTransparent)
			continue;

		if (isTransparent) {
			glEnable(GL_BLEND);
			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
			glAlphaFunc(GL_GREATER, 0.0);
		} else {
			glDisable(GL_BLEND);
		}

		if (g.Mat[s->MatIndex].Color[0] == 60 &&
		    g.Mat[s->MatIndex].Color[1] == 255 &&
		    g.Mat[s->MatIndex].Color[2] == 0) {
			glColor4ubv(carcol1);
		} else if (g.Mat[s->MatIndex].Color[0] == 255 &&
		    g.Mat[s->MatIndex].Color[1] == 0 &&
		    g.Mat[s->MatIndex].Color[2] == 175) {
			glColor4ubv(carcol2);
		} else {
			glColor4ubv(g.Mat[s->MatIndex].Color);
		}

		if (g.FaceType == 1) {
			glBegin(GL_TRIANGLE_STRIP);
			for (k = 0; k < s->numIndices-2; k++) {
			for (m = 0; m < 3; m++) {
				i = s->Indices[k+m];
				if ((g.Flags & rpGEOMETRYTEXTURED) ||
				     (g.Flags & rpGEOMETRYTEXTURED2))
					glMultiTexCoord2fv(GL_TEXTURE0, uv[i]);

				if (g.Flags & rpGEOMETRYNORMALS)
					glNormal3fv(norm[i]);
				else
					glNormal3fv(s->Normals[k]);
				glVertex3fv(vert[i]);
			}
			}
		} else {
			glBegin(GL_TRIANGLES);
			for (k = 0; k < s->numIndices; k++) {
				i = s->Indices[k];
				if ((g.Flags & rpGEOMETRYTEXTURED) ||
				     (g.Flags & rpGEOMETRYTEXTURED2))
					glMultiTexCoord2fv(GL_TEXTURE0, uv[i]);

				if (g.Flags & rpGEOMETRYNORMALS)
					glNormal3fv(norm[i]);
				else
					glNormal3fv(s->Normals[k/3]);
				glVertex3fv(vert[i]);
			}
		}
		glEnd();
	}
}

void CGlDff::DrawAtomic(int index)
{
	glCallList(DLarray[index]);

/*
	int i, g, f;
	
	for (i = 0; i < Clp->numAtomics; i++) {
		g = Clp->Atm[i].GeoIndex;
		f = Clp->Atm[i].FrmIndex;

		glPushMatrix();
		TransformFrame(Clp->Frm, f);
		DrawGeometry(Clp->Geo[g], txd, 0);
		DrawGeometry(Clp->Geo[g], txd, 1);
		glPopMatrix();
	}
*/
}

GLuint *CGlDff::CreateDLarray()
{
	DLarray = new GLuint[RpClumpGetNumAtomics(Clump)];
	if (DLarray == 0)
		return NULL;
	else
		return DLarray;
}

void CGlDff::DestroyDLarray()
{
	delete[] DLarray;
}

GLuint *CGlDff::GetDLarray()
{
	return DLarray;
}

void CGlDff::CreateDisplayLists(CGlTxd &txd)
{
	RwUInt32 i;
	RwInt32 g, f;
	GLuint *DLarray;
	GLuint DL;

	if ((DLarray = CreateDLarray()) == NULL)
		exit(11);
	DL = glGenLists(RpClumpGetNumAtomics(Clump));

	for (i = 0; i < RpClumpGetNumAtomics(Clump); i++) {
		DLarray[i] = DL+i;
		g = Clump->Atm[i].GeoIndex;
		f = Clump->Atm[i].FrmIndex;

		glNewList(DL+i, GL_COMPILE);
			glPushMatrix();
			TransformFrame(Clump->Frm, f);
			DrawGeometry(Clump->Geo[g], txd, 0);
			DrawGeometry(Clump->Geo[g], txd, 1);
			glPopMatrix();
		glEndList();
	}
	cerr << "Created Display lists\n";
}

void CGlDff::DestroyDisplayLists()
{
	RwUInt32 i;

	for (i = 0; i < RpClumpGetNumAtomics(Clump); i++)
		glDeleteLists(DLarray[i], 1);
	DestroyDLarray();
}

void CGlDff::SetClump(RpClump *c)
{
	Clump = c;
}

RpClump *CGlDff::GetClump()
{
	return Clump;
}

