#ifdef WIN32
  #include <windows.h>
#endif
#ifdef GLEW
  #include <GL/glew.h>
#endif
#include <iostream>
#include <GL/gl.h>
#include <GL/glu.h>

#include "dffviewer.h"

#ifdef GLX
  #include "glx.h"
#else
  #include "glut.h"
#endif

using namespace std;

char progname[30];

GLfloat camx = 0.0f, camy = 0.0f, camz = -43.0f;
GLfloat camyaw = -66.0f, campitch = 0.0f, camroll = -120.0f;

//GLfloat lgtdif[] = { 0.7f, 0.7f, 0.7f, 1.0f };
GLfloat lgtdif[] = { 0.8f, 0.8f, 0.8f, 1.0f };
GLfloat lgtamb[] = { 1.0f, 1.0f, 1.0f, 1.0f };
//GLfloat lgtamb[] = { 0.4f, 0.4f, 0.4f, 1.0f };
//GLfloat lgtamb[] = { 0.3f, 0.3f, 0.3f, 1.0f };
GLfloat lgtspe[] = { 1.0f, 1.0f, 1.0f, 1.0f };
GLfloat lgtpos[] = { 0.0f, 0.0f, 2.0f, 1.0f };

void RenderScene()
{
	RwUInt32 i;

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();

	glTranslatef(camx, camy, camz);
	glRotatef(camyaw, 1.0f, 0.0f, 0.0f);
	glRotatef(campitch, 0.0f, 1.0f, 0.0f);
	glRotatef(camroll, 0.0f, 0.0f, 1.0f);

	for (i = 0; i < RpClumpGetNumAtomics(mdl.GetClump()); i++) {
		if (UseDisplayLists)
			mdl.DrawAtomicDL(i);
		else
			mdl.DrawAtomic(i);
	}

	SwapBuffers();
}

void Init()
{
//	GLfloat zPlane[] = {0.0f, 0.0f, 1.0f, 0.0f};
	int w, h;

	w = getWidth();
	h = getHeight();

#ifdef GLEW
	}
		GLenum err;
		err = glewInit();
		if (err != GLEW_OK) {
			cerr << "Error with GLEW\n";
			exit(1);
		}
	}
#endif
	glClearColor(0.6f, 0.6f, 0.6f, 1.0f);
	glClearDepth(1.0);
	glDepthFunc(GL_LESS);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_ALPHA_TEST);
	glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
	glShadeModel(GL_SMOOTH);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(45.0f, (GLfloat) w/(GLfloat)h, 0.1f, 2000.0f);

	glMatrixMode(GL_MODELVIEW);

	/* Set up lighting */
	glEnable(GL_LIGHTING);
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lgtamb);
//	glLightfv(GL_LIGHT0, GL_AMBIENT, lgtamb);
//	glLightfv(GL_LIGHT0, GL_DIFFUSE, lgtdif);
//	glLightfv(GL_LIGHT0, GL_SPECULAR, lgtspe);
//	glEnable(GL_LIGHT0);
	glEnable(GL_COLOR_MATERIAL);
	glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE);
//	glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE);
//	glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);

	/* Set up texture units */
	glActiveTexture(GL_TEXTURE0);
	glEnable(GL_TEXTURE_2D);
	glActiveTexture(GL_TEXTURE1);
	glEnable(GL_TEXTURE_2D);

	/* Set up texture generation for reflection maps */
	glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
	glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
//	glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
//	glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
//	glTexGenfv(GL_S, GL_EYE_PLANE, zPlane);
//	glTexGenfv(GL_T, GL_EYE_PLANE, zPlane);

	glEnable(GL_TEXTURE_GEN_S);
	glEnable(GL_TEXTURE_GEN_T);
	glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_ADD);
	glActiveTexture(GL_TEXTURE0);

	if (UseTextures)
		mdl.LoadTxd();
	if (UseDisplayLists)
		mdl.CreateDisplayLists();
}

void DeInit()
{
	if (UseDisplayLists)
		mdl.DestroyDisplayLists();
	if (UseTextures)
		mdl.UnloadTxd();
}
