#include <iostream>
#include <cmath>
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glx.h>
#include <X11/extensions/xf86vmode.h>
#include <X11/keysym.h>

#include "dffviewer.h"
#include "glx.h"

#define SENSITIVITY 500

using namespace std;

int isLDown;
int isRDown;
int ox, oy;
int mx, my;
float camphi, camtheta;

GL_Window window;
Bool done;

int attrListSgl[] = {
	GLX_RGBA,
	GLX_RED_SIZE, 4,
	GLX_GREEN_SIZE, 4,
	GLX_BLUE_SIZE, 4,
	GLX_DEPTH_SIZE, 16,
	None
};

int attrListDbl[] = {
	GLX_RGBA, GLX_DOUBLEBUFFER,
	GLX_RED_SIZE, 4,
	GLX_GREEN_SIZE, 4,
	GLX_BLUE_SIZE, 4,
	GLX_DEPTH_SIZE, 16,
	None
};

void Resize(int width, int height)
{
	glViewport(0, 0, (GLsizei)(width), (GLsizei)(height));
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(45.0f, (GLfloat)(width)/(GLfloat)(height), 1.0f, 100.0f);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

bool CreateWindowGL(char *progname)
{
	XVisualInfo *vi;
	Colormap cmap;
	int dpyWidth, dpyHeight;
	int i;
	int glxMajorVersion, glxMinorVersion;
	int vidModeMajorVersion, vidModeMinorVersion;
	XF86VidModeModeInfo **modes;
	int modeNum;
	int bestMode;
	Atom wmDelete;
	Window winDummy;
	unsigned int borderDummy;
	int x, y;

	bestMode = 0;

	window.init.dpy = XOpenDisplay(NULL);
	window.init.screen = DefaultScreen(window.init.dpy);
	XF86VidModeQueryVersion(window.init.dpy, &vidModeMajorVersion,
	  &vidModeMinorVersion);

	XF86VidModeGetAllModeLines(window.init.dpy, window.init.screen,
	  &modeNum, &modes);
	window.init.deskMode = *modes[0];

	for (i = 0; i < modeNum; i++) {
		if ((modes[i]->hdisplay == window.init.width) &&
		  (modes[i]->vdisplay == window.init.height))
			bestMode = i;
	}

	vi = glXChooseVisual(window.init.dpy, window.init.screen,attrListDbl);
	if (vi == NULL) {
		vi = glXChooseVisual(window.init.dpy,
		  window.init.screen, attrListSgl);
		window.init.doubleBuffered = false;
	} else {
		window.init.doubleBuffered = true;
	}

	glXQueryVersion(window.init.dpy, &glxMajorVersion, &glxMinorVersion);

	window.init.ctx = glXCreateContext(window.init.dpy, vi, 0, GL_TRUE);

	cmap = XCreateColormap(window.init.dpy,
	  RootWindow(window.init.dpy, vi->screen),
	  vi->visual, AllocNone);
	window.init.attr.colormap = cmap;
	window.init.attr.border_pixel = 0;

	if (window.init.isFullScreen) {
		XF86VidModeSwitchToMode(window.init.dpy, window.init.screen,
		  modes[bestMode]);

		XF86VidModeSetViewPort(window.init.dpy,
		  window.init.screen, 0, 0);

		dpyWidth = modes[bestMode]->hdisplay;
		dpyHeight = modes[bestMode]->vdisplay;
		XFree(modes);

		window.init.attr.override_redirect = true;

		window.init.attr.event_mask =
		  ExposureMask | KeyPressMask | KeyReleaseMask |
		  ButtonPressMask | ButtonReleaseMask | StructureNotifyMask |
		  PointerMotionMask;

		window.init.win =
		  XCreateWindow(window.init.dpy,
		  RootWindow(window.init.dpy, vi->screen),
		  0, 0, dpyWidth, dpyHeight, 0, vi->depth, InputOutput,
		  vi->visual, CWBorderPixel | CWColormap | CWEventMask |
		  CWOverrideRedirect, &window.init.attr);

		XWarpPointer(window.init.dpy, None, window.init.win,
		  0, 0, 0, 0, 0, 0);

		XMapRaised(window.init.dpy, window.init.win);

		XGrabKeyboard(window.init.dpy, window.init.win,
		  True, GrabModeAsync, GrabModeAsync, CurrentTime);

		XGrabPointer(window.init.dpy, window.init.win, True,
		  ButtonPressMask, GrabModeAsync, GrabModeAsync,
		  window.init.win, None, CurrentTime);
	} else {
		window.init.attr.event_mask = ExposureMask | KeyPressMask |
		  KeyReleaseMask | ButtonPressMask | ButtonReleaseMask |
		  StructureNotifyMask | PointerMotionMask;

		window.init.win =
		  XCreateWindow(window.init.dpy,
		  RootWindow(window.init.dpy, vi->screen),
		  0, 0, window.init.width, window.init.height,
		  0, vi->depth, InputOutput, vi->visual,
		  CWBorderPixel | CWColormap | CWEventMask,
		  &window.init.attr);

		wmDelete =
		  XInternAtom(window.init.dpy, "WM_DELETE_WINDOW", True);

		XSetWMProtocols(window.init.dpy, window.init.win,
		  &wmDelete, 1);

		XSetStandardProperties(window.init.dpy, window.init.win,
		  window.init.title, window.init.title, None, NULL,
		  0, NULL);

		XMapRaised(window.init.dpy, window.init.win);
	}

	glXMakeCurrent(window.init.dpy, window.init.win, window.init.ctx);
	XGetGeometry(window.init.dpy, window.init.win, &winDummy, &x, &y,
	  &window.init.width, &window.init.height, &borderDummy,
	  &window.init.depth);

	Init();

	return true;
}

void KillGlWindow()
{
	DeInit();

	if (window.init.ctx) {
		glXMakeCurrent(window.init.dpy, None, NULL);
		glXDestroyContext(window.init.dpy, window.init.ctx);
	}

	if (window.init.isFullScreen) {
		XF86VidModeSwitchToMode( window.init.dpy,
		  window.init.screen, &window.init.deskMode);
		XF86VidModeSetViewPort(window.init.dpy,
		  window.init.screen, 0, 0);
	}
	XCloseDisplay(window.init.dpy);
}

void SwapBuffers()
{
	glXSwapBuffers(window.init.dpy, window.init.win);
}

void Keypress(int key)
{
	switch(key) {
	case XK_Up:
		camyaw += 2.0f;
		break;
	case XK_Down:
		camyaw -= 2.0f;
		break;
	case XK_Left:
		camroll -= 2.0f;
		break;
	case XK_Right:
		camroll += 2.0f;
		break;
	case XK_Page_Up:
		camz += 0.7f;
		break;
	case XK_Page_Down:
		camz -= 0.7f;
		break;
	case XK_Escape:
                done = true;
                break;
	case XK_F1:
		KillGlWindow();
		window.init.isFullScreen = !window.init.isFullScreen;
		CreateWindowGL(progname);
	default:
		break;
	}
}

int getWidth()
{
	return window.init.width;
}

int getHeight()
{
	return window.init.height;
}


void StartOgl(int argc, char *argv[])
{
	XEvent event;
	Keys keys;
	KeySym key;

	window.keys = &keys;
	window.init.title = progname;
	window.init.width = 640;
	window.init.height = 480;
	window.init.depth = 32;
	window.init.isFullScreen = false;
	done = false;

	if (CreateWindowGL(progname) == false)
		exit(1);

	while (!done) {
		while (XPending(window.init.dpy) > 0) {
			XNextEvent( window.init.dpy, &event);
			switch (event.type) {
			case Expose:
				if (event.xexpose.count != 0)
					break;
				RenderScene();
				break;
			case ConfigureNotify:
				if (((unsigned int) event.xconfigure.width !=
				     window.init.width) ||
				    ((unsigned int) event.xconfigure.height !=
				     window.init.height)) {
					window.init.width =
					  event.xconfigure.width;
					window.init.height =
					  event.xconfigure.height;
					Resize(window.init.width,
					  window.init.height);
					}
					break;
			case ButtonPress:
				switch(event.xbutton.button) {
				case 1:
					mx = event.xbutton.x;
					my = event.xbutton.y;
					isLDown = 1;
					break;
				case 3:
					isRDown = 1;
					break;
				default:
					break;
				}
				break;
			case ButtonRelease:
				switch(event.xbutton.button) {
				case 1:
					isLDown = 0;
					break;
				case 3:
					isRDown = 0;
					break;
				case 5:
					break;
				case 6:
					break;
				default:
					break;
				}
				break;
			case MotionNotify:
				if (isLDown) {
					ox = mx;
					oy = my;
					mx = event.xbutton.x;
					my = event.xbutton.y;
					camtheta += (float)(ox-mx) /
					  (float)SENSITIVITY;
					camphi += (float)(oy-my) /
					  (float)SENSITIVITY;
//					CamCalcVector();
				}
				break;
			case KeyPress:
				key = XLookupKeysym(&event.xkey, 0);
				Keypress(key);
				break;
			case KeyRelease:
				break;
			case ClientMessage:
				if (*XGetAtomName(window.init.dpy,
				  event.xclient.message_type)==*"WM_PROTOCOLS")
					done = true;
				break;
			default:
				break;
			}
		}
		RenderScene();
	}

	KillGlWindow();
}
