#include <iostream>
#include "dffviewer.h"
using namespace std;

GLubyte carcol1[] = { 234, 205, 81, 255 };
GLubyte carcol2[] = { 80, 40, 200, 255 };

/* Clump */

void Model::TransformFrame(RwFrame *f, int which)
{
	GLfloat transmat[16];

	if (f[which].Parent != -1)
		TransformFrame(f, f[which].Parent);

	transmat[0] = f[which].Rotation[0][0];
	transmat[1] = f[which].Rotation[0][1];
	transmat[2] = f[which].Rotation[0][2];
	transmat[3] = 0;

	transmat[4] = f[which].Rotation[1][0];
	transmat[5] = f[which].Rotation[1][1];
	transmat[6] = f[which].Rotation[1][2];
	transmat[7] = 0;

	transmat[8] = f[which].Rotation[2][0];
	transmat[9] = f[which].Rotation[2][1];
	transmat[10] = f[which].Rotation[2][2];
	transmat[11] = 0;

	transmat[12] = f[which].Position[0];
	transmat[13] = f[which].Position[1];
	transmat[14] = f[which].Position[2];
	transmat[15] = 1;

	glMultMatrixf(transmat);
}

void Model::DrawGeometry(RpGeometry &g, int drawtransp)
{
	RwUInt32 i, j, k, m;
	int isTransparent;
	RpMesh *s;
	RwBool hasAlpha;
	Vector3f *vert, *norm;
	Vector2f *uv;
	Color4b *color;
	RwUInt8 *meshcol;
	Color4b avcolor;

	avcolor[3] = 255;
	for (j = 0; j < g.numSplits; j++) {
		s = &g.Split[j];
		vert = g.Vertex;
		norm = g.Normal;
		uv = g.UV;
		color = g.VColor;

		isTransparent = 0;
		if (UseTextures) {
			glActiveTexture(GL_TEXTURE1);
			glBindTexture(GL_TEXTURE_2D, 0);
			glActiveTexture(GL_TEXTURE0);
			glBindTexture(GL_TEXTURE_2D, 0);

			/* if there is a diffuse map, use it */
			if (g.Mat[s->MatIndex].HasTex) {
				hasAlpha = BindTex(&g.Mat[s->MatIndex].Tex);
				if (hasAlpha)
					isTransparent = 1;
			}

			/* if there is a reflection map, use it */
			if (g.Mat[s->MatIndex].HasRef) {
				glActiveTexture(GL_TEXTURE1);
				BindTex(&g.Mat[s->MatIndex].Reflection);
				glActiveTexture(GL_TEXTURE0);
			}
		}

		if (g.Mat[s->MatIndex].Color[3] != 255)
			isTransparent = 1;

		if (drawtransp != isTransparent)
			continue;

		if (isTransparent) {
			glEnable(GL_BLEND);
			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
			glAlphaFunc(GL_GREATER, 0.0);
		} else {
			glDisable(GL_BLEND);
		}

		if (g.Mat[s->MatIndex].Color[0] == 60 &&
		    g.Mat[s->MatIndex].Color[1] == 255 &&
		    g.Mat[s->MatIndex].Color[2] == 0)
			meshcol = carcol1;
		else if (g.Mat[s->MatIndex].Color[0] == 255 &&
		    g.Mat[s->MatIndex].Color[1] == 0 &&
		    g.Mat[s->MatIndex].Color[2] == 175)
			meshcol = carcol2;
		else
			meshcol = g.Mat[s->MatIndex].Color;
		glColor4ubv(meshcol);
		

		if (g.FaceType == 1) {
			glBegin(GL_TRIANGLE_STRIP);
			for (k = 0; k < s->numIndices-2; k++) {
			for (m = 0; m < 3; m++) {
				i = s->Indices[k+m];
				if ((g.Flags & rpGEOMETRYTEXTURED) ||
				     (g.Flags & rpGEOMETRYTEXTURED2))
					glMultiTexCoord2fv(GL_TEXTURE0, uv[i]);

				if (g.Flags & rpGEOMETRYPRELIT) {
					avcolor[0]=(meshcol[0]+color[i][0])/2;
					avcolor[1]=(meshcol[1]+color[i][1])/2;
					avcolor[2]=(meshcol[2]+color[i][2])/2;
					glColor4ubv(avcolor);
				}

				if (g.Flags & rpGEOMETRYNORMALS)
					glNormal3fv(norm[i]);
//				else
//					glNormal3fv(s->Normals[k]);
				glVertex3fv(vert[i]);
			}
			}
		} else {
			glBegin(GL_TRIANGLES);
			for (k = 0; k < s->numIndices; k++) {
				i = s->Indices[k];
				if ((g.Flags & rpGEOMETRYTEXTURED) ||
				     (g.Flags & rpGEOMETRYTEXTURED2))
					glMultiTexCoord2fv(GL_TEXTURE0, uv[i]);
				if (g.Flags & rpGEOMETRYPRELIT) {
					avcolor[0]=(meshcol[0]+color[i][0])/2;
					avcolor[1]=(meshcol[1]+color[i][1])/2;
					avcolor[2]=(meshcol[2]+color[i][2])/2;
					glColor4ubv(avcolor);
				}

				if (g.Flags & rpGEOMETRYNORMALS)
					glNormal3fv(norm[i]);
//				else
//					glNormal3fv(s->Normals[k/3]);
				glVertex3fv(vert[i]);
			}
		}
		glEnd();
	}
}

void Model::DrawAtomicDL(int index)
{
	glCallList(DLarray[index]);
}

void Model::DrawAtomic(int index)
{
	int i, g, f;
	
	for (i = 0; i < Clump->numAtomics; i++) {
		g = Clump->Atm[i].GeoIndex;
		f = Clump->Atm[i].FrmIndex;

		glPushMatrix();
		TransformFrame(Clump->Frm, f);
		DrawGeometry(Clump->Geo[g], 0);
		DrawGeometry(Clump->Geo[g], 1);
		glPopMatrix();
	}
}

void Model::CreateDisplayLists()
{
	RwUInt32 i;
	RwInt32 g, f;
	GLuint DL;

	cout << "Creating Display lists\n";

	DLarray = new GLuint[RpClumpGetNumAtomics(Clump)];
	DL = glGenLists(RpClumpGetNumAtomics(Clump));

	if (DL == 0) {
		cerr << "No Display Lists could be allocated\n";
		exit(1);
	}

	for (i = 0; i < RpClumpGetNumAtomics(Clump); i++) {
		DLarray[i] = DL+i;
		g = Clump->Atm[i].GeoIndex;
		f = Clump->Atm[i].FrmIndex;

		glNewList(DL+i, GL_COMPILE);
			glPushMatrix();
			TransformFrame(Clump->Frm, f);
			DrawGeometry(Clump->Geo[g], 0);
			DrawGeometry(Clump->Geo[g], 1);
			glPopMatrix();
		glEndList();
	}
	cout << "Created Display lists\n";
}

void Model::DestroyDisplayLists()
{
	RwUInt32 i;

	cout << "Destroying Display lists\n";

	for (i = 0; i < RpClumpGetNumAtomics(Clump); i++)
		glDeleteLists(DLarray[i], 1);
	delete[] DLarray;
	DLarray = NULL;

	cout << "Destroyed Display lists\n";
}

void Model::SetClump(RpClump *c)
{
	Clump = c;
}

RpClump *Model::GetClump()
{
	return Clump;
}

/* Texture Dictionary */

RwBool Model::BindTex(RwTexture *tex)
{
	RwInt32 index;

	if ((index = GetTexIndex(tex->Name)) != -1)
		glBindTexture(GL_TEXTURE_2D, texid[index]);
	else
		glBindTexture(GL_TEXTURE_2D, 0);
	return TexDict->texture[index].HasAlpha;
}

void Model::LoadTxd()
{
	RwUInt32 i;

	cout << "Loading Textures\n";
	texid = new GLuint[TexDict->numTextures];
	glGenTextures(TexDict->numTextures, texid);
	for (i = 0; i < TexDict->numTextures; i++) {
		glBindTexture(GL_TEXTURE_2D, texid[i]);

		glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);

		if (TexDict->texture[i].depth == 32) {
			glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA,
			             TexDict->texture[i].width,
			             TexDict->texture[i].height,
			             0, GL_RGBA, GL_UNSIGNED_BYTE,
			             TexDict->texture[i].Data_a);
		} else {
			cerr << "Can't handle non 32bit textures\n";
			exit(1);
		}
	}
	cout << "Textures Loaded\n";
}

void Model::UnloadTxd()
{
	RwUInt32 i;

	cout << "Unloading Textures\n";

	for (i = 0; i < TexDict->numTextures; i++)
		glDeleteTextures(1, &texid[i]);
	delete[] texid;
	texid = NULL;

	cout << "Textures unloaded\n";
}

RwInt32 Model::GetTexIndex(RwChar *Name)
{
	RwUInt32 i;

	for (i = 0; i < TexDict->numTextures; i++)
		if (strcasecmp(TexDict->texture[i].Name, Name) == 0)
			return i;
	return -1;
}

void Model::SetRwTexDictionary(RwTexDictionary *td)
{
	TexDict = td;
}

RwTexDictionary *Model::GetRwTexDictionary()
{
	return TexDict;
}

Model::Model()
{
	TexDict = NULL;
	texid = NULL;
	Clump = NULL;
	DLarray = NULL;
}

Model::~Model()
{
	if (TexDict != NULL) {
		RwTexDictionaryDestroy(TexDict);
		TexDict = NULL;
	}
	if (Clump != NULL) {
		RpClumpDestroy(Clump);
		Clump = NULL;
	}
	if (texid != NULL) {
		UnloadTxd();
	}
	if (DLarray != NULL) {
		DestroyDisplayLists();
	}
}
