#include "rw.h"

#ifndef RW_DFF_H
#define RW_DFF_H

enum RwCorePluginID {
	rwID_NAOBJECT       = 0x0,
	rwID_STRUCT         = 0x1,
	rwID_STRING         = 0x2,
	rwID_EXTENSION      = 0x3,
	rwID_TEXTURE        = 0x6,
	rwID_MATERIAL       = 0x7,
	rwID_MATLIST        = 0x8,
	rwID_FRAMELIST      = 0xE,
	rwID_GEOMETRY       = 0xF,
	rwID_CLUMP          = 0x10,
	rwID_ATOMIC         = 0x14,
	rwID_TEXDICTIONARY  = 0x16,
	rwID_GEOMETRYLIST   = 0x1A,
	rwID_RIGHTTORENDER  = 0x1F,

	rwID_MORPH           = 0x105,
	rwID_SKYMIPMAP       = 0x110,
	rwID_SKIN            = 0x116,
	rwID_PARTICLES       = 0x118,
	rwID_HANIM           = 0x11E,
	rwID_MATERIALEFFECTS = 0x120,
	rwID_ADCPLG          = 0x134,
	rwID_BINMESH         = 0x50E,
	rwID_NATIVEDATA      = 0x510,

	rwID_SPECULARMAT      = 0x253F2F6,
	rwID_NIGHTVERTEXCOLOR = 0x253F2F9,
	rwID_COLLISIONMODEL   = 0x253F2FA,
	rwID_REFLECTIONMAT    = 0x253F2FC,
	rwID_MESHEXTENSION    = 0x253F2FD,
	rwID_FRAME            = 0x253F2FE,
};

enum {
	rpGEOMETRYTRISTRIP   = 0x01,
	rpGEOMETRYPOSITIONS  = 0x02,
	rpGEOMETRYTEXTURED   = 0x04,
	rpGEOMETRYPRELIT     = 0x08,
	rpGEOMETRYNORMALS    = 0x10,
	rpGEOMETRYLIGHT      = 0x20,
	rpGEOMETRYMODULATEMATERIALCOLOR  = 0x40,
	rpGEOMETRYTEXTURED2  = 0x80,
};

enum {
	GTA3_1 = 0x00000310,
	GTA3_2 = 0x0800FFFF,
	VCPS2  = 0x0C02FFFF,
	SA     = 0x1803FFFF
};

#define VCNORMSCALE (float) 1/128
#define SAVERTSCALE1 (float) 1/128 /* normally used */
#define SAVERTSCALE2 (float) 1/1024 /* used by cars and peds */
#define SAUVSCALE   (float) 1/4096
#define SANORMSCALE (float) 1/128

namespace Rw {

typedef byte Color4b[4];
typedef uint16 Face4i[4];
typedef float Vector3f[3];
typedef float Vector2f[2];

typedef struct RpAtomic RpAtomic;
struct RpAtomic {
	int FrmIndex;
	int GeoIndex;
	int Unknown0;
	int Unknown1;
};

typedef struct RpMesh RpMesh;
struct RpMesh {
	uint32 numIndices;
	uint32 MatIndex;

	uint32 *Indices;
	Vector3f *Normals;
};

typedef struct RpTexture RpTexture;
struct RpTexture {
	uint16 FilterFlags;
	uint16 Unknown;
	char *Tex;
	char *Alpha;
};

typedef struct RpMaterial RpMaterial;
struct RpMaterial {
	int Flags;
	byte Color[4];
	int Unused;
	uint32 HasTex;
	float SurfaceProps[3];


	uint32 TexRead;
	RpTexture Tex;

	uint32 HasRef;
	RpTexture Reflection;

	/* San Andreas only */
	float RefImageAmount[4];
	float RefIntensity;

	float SpecLevel;
	char *SpecName;
};

typedef struct RpGeometry RpGeometry;
struct RpGeometry {
	uint16 Flags;
	byte numUVs;
	byte isPs2;
	uint32 numTriangles;
	uint32 numVertices;
	uint32 numMorphTargets;

	Color4b *VColor;
	Vector2f *UV;
	Vector2f *UV2;
	Face4i *Face;
	Vector3f *Vertex;
	Vector3f *Normal;

	/* Bounding shpere */
	float Bs[4];
	int BsPos;
	int BsNor;

	uint32 numMaterials;
	RpMaterial *Mat;

	uint32 FaceType;
	uint32 numSplits;
	uint32 numIndices;
	RpMesh *Split;
};

typedef struct RwFrame RwFrame;
struct RwFrame {
	float Rotation[3][3];
	float Position[3];
	int Parent;
	uint32 Unknown;

	char *Name;
};

typedef struct RpClump RpClump;
struct RpClump {
	int32 numAtomics;
	RpAtomic *Atm;

	int32 numFrames;
	RwFrame *Frm;

	int32 numGeometries;
	RpGeometry *Geo;

	/* Not used */
	int32 numLights;
	int32 numCameras;
};


struct ps2split {
	uint32 vertexc;

	byte **vc;
	float **uv;
	float **uv2;
	float **vertices;
	float **normals;

	uint16 *flags;
	uint32 extraverts;
};

class CDff {
public:
	RpClump *Clp;
	uint32 numClumps;
	void ParseSection(RpClump&, CHeader&, uint32, istream&);
	void ReadSection(RpClump&, CHeader&, istream&);
	void ReadStruct(RpClump&, CHeader&, uint32, istream&);
	void ReadFrame(RwFrame&, uint32, istream&);
	void ReadMatfx(RpClump&, CHeader&, uint32, istream&);
	void ReadNativeDataVcPs2(RpGeometry&, istream&);
	void ReadNativeDataSaPs2(RpGeometry&, istream&);
	int  IsSaCompression(RpGeometry&, istream&);
	void MakeNormals(RpGeometry&, RpMesh&);
	void CalcNormal(Vector3f&, Vector3f&, Vector3f&, Vector3f&);
	void Ps2ToPc(RpGeometry&, struct ps2split *);
	int  IsSameVert(float *, float *);
	float GetSaVertScale(RpGeometry&);
	void ReadFromFile(istream&);
	void ReadFromFile(istream&, uint32, uint32);
	CDff(void);
	~CDff(void);
};

}

#endif
