#include <iostream>
#include <fstream>
#include "rw.h"
using namespace std;

uint32 ReadDword(istream &file)
{
	uint32 tmp;
	file.read((char*)&tmp, 4);
	return tmp;
}

uint32 PeekDword(istream &file)
{
	uint32 tmp;
	file.read((char*)&tmp, 4);
	file.seekg(-4, ios::cur);
	return tmp;
}

uint16 ReadWord(istream &file)
{
	uint16 tmp;
	file.read((char*)&tmp, 2);
	return tmp;
}

uint8 ReadByte(istream &file)
{
	uint8 tmp;
	file.read((char*)&tmp, 1);
	return tmp;
}

float ReadFloat(istream &file)
{
	float tmp;
	file.read((char*)&tmp, 4);
	return tmp;
}

void WriteDword(ostream &file, uint32 dword)
{
	file.write((char*)&dword, 4);
}

void WriteWord(ostream &file, uint16 word)
{
	file.write((char*)&word, 2);
}

void WriteByte(ostream &file, uint8 byte)
{
	file.write((char*)&byte, 1);
}

void WriteFloat(ostream &file, float fl)
{
	file.write((char*)&fl, 4);
}

char *sections[] = { "None", "Struct", "String", "Extension", "Unknown",
	"Camera", "Texture", "Material", "Material List", "Atomic Section",
	"Plane Section", "World", "Spline", "Matrix", "Frame List",
	"Geometry", "Clump", "Unknown", "Light", "Unicode String", "Atomic",
	"Texture Native", "Texture Dictionary", "Animation Database",
	"Image", "Skin Animation", "Geometry List", "Anim Animation",
	"Team", "Crowd", "Delta Morph Animation", "Right To Render",
	"MultiTexture Effect Native", "MultiTexture Effect Dictionary",
	"Team Dictionary", "Platform Independet Texture Dictionary",
	"Table of Contents", "Particle Standard Global Data", "AltPipe",
	"Platform Independet Peds", "Patch Mesh", "Chunk Group Start",
	"Chunk Group End", "UV Animation Dictionary", "Coll Tree"
};

/* From 0x0101 through 0x0135 */
char *toolkitsections0[] = { "Metrics PLG", "Spline PLG", "Stereo PLG",
	"VRML PLG", "Morph PLG", "PVS PLG", "Memory Leak PLG", "Animation PLG",
	"Gloss PLG", "Logo PLG", "Memory Info PLG", "Random PLG",
	"PNG Image PLG", "Bone PLG", "VRML Anim PLG", "Sky Mipmap Val",
	"MRM PLG", "LOD Atomic PLG", "ME PLG", "Lightmap PLG",
	"Refine PLG", "Skin PLG", "Label PLG", "Particles PLG", "GeomTX PLG",
	"Synth Core PLG", "STQPP PLG",
	"Part PP PLG", "Collision PLG", "HAnim PLG", "User Data PLG",
	"Material Effects PLG", "Particle System PLG", "Delta Morph PLG",
	"Patch PLG", "Team PLG", "Crowd PP PLG", "Mip Split PLG",
	"Anisotrophy PLG", "Not used", "GCN Material PLG", "Geometric PVS PLG",
	"XBOX Material PLG", "Multi Texture PLG", "Chain PLG", "Toon PLG",
	"PTank PLG", "Particle Standard PLG", "PDS PLG", "PrtAdv PLG",
	"Normal Map PLG", "ADC PLG", "UV Animation PLG"
};

/* From 0x0180 through 0x01c1 */
char *toolkitsections1[] = {
	"Character Set PLG", "NOHS World PLG", "Import Util PLG",
	"Slerp PLG", "Optim PLG", "TL World PLG", "Database PLG",
	"Raytrace PLG", "Ray PLG", "Library PLG",
	"Not used", "Not used", "Not used", "Not used", "Not used", "Not used",
	"2D PLG", "Tile Render PLG", "JPEG Image PLG", "TGA Image PLG",
	"GIF Image PLG", "Quat PLG", "Spline PVS PLG", "Mipmap PLG",
	"MipmapK PLG", "2D Font", "Intersection PLG", "TIFF Image PLG",
	"Pick PLG", "BMP Image PLG", "RAS Image PLG", "Skin FX PLG",
	"VCAT PLG", "2D Path", "2D Brush", "2D Object", "2D Shape", "2D Scene",
	"2D Pick Region", "2D Object String", "2D Animation PLG",
	"2D Animation",
	"Not used", "Not used", "Not used", "Not used", "Not used", "Not used",
	"2D Keyframe", "2D Maestro", "Barycentric",
	"Platform Independent Texture Dictionary TK", "TOC TK", "TPL TK",
	"AltPipe TK", "Animation TK", "Skin Split Tookit", "Compressed Key TK",
	"Geometry Conditioning PLG", "Wing PLG", "Generic Pipeline TK",
	"Lightmap Conversion TK", "Filesystem PLG", "Dictionary TK",
	"UV Animation Linear", "UV Animation Parameter"
};

char *RSsections[] = { "Unused 1", "Unused 2", "Unused 3",
	"Pipeline Set", "Unused 5", "Unused 6", "Specular Material",
	"Unused 8", "2dfx", "Night Vertex Colors", "Collision Model",
	"Unused 12", "Reflection Material", "Mesh Extension", "Frame",
	"Unused 16"
};

void CHeader::Write(ostream& file)
{
	WriteDword(file, Type);
	WriteDword(file, Size);
	WriteDword(file, Version);
}

void CHeader::Read(istream& File)
{
	File.read((char*)&Type, sizeof(int));
	if (File.eof())
		Type = 0;
	File.read((char*)&Size, sizeof(int));
	if (File.eof())
		Type = 0;
	File.read((char*)&Version, sizeof(int));
	if (File.eof())
		Type = 0;
}

char *CHeader::GetSectionName(uint32 t)
{
	switch (t) {
	case 0x50E:
		return "Bin Mesh PLG";
		break;
	case 0x510:
		return "Native Data PLG";
		break;
	case 0xF21E:
		return "ZModeler Lock";
		break;
	default:
		break;
	}
	
	if (t <= 45)
		return sections[t];
	else if (t <= 0x0253F2FF && t >= 0x0253F2F0)
		return RSsections[t-0x0253F2F0];
	else if (t <= 0x0135 && t >= 0x0101)
		return toolkitsections0[t-0x0101];
	else if (t <= 0x0180 && t >= 0x01c1)
		return toolkitsections1[t-0x0180];
	else
		return "Unknown";
}

inline void CHeader::SetType(uint32 t)
{
	Type = t;
}

inline void CHeader::SetSize(uint32 s)
{
	Size = s;
}

inline void CHeader::SetVersion(uint32 v)
{
	Version = v;
}

inline void CHeader::SetEverything(uint32 t, uint32 s, uint32 v)
{
	Type = t;
	Size = s;
	Version = v;
}

uint32 CHeader::GetType()
{
	return Type;
}

uint32 CHeader::GetSize()
{
	return Size;
}

uint32 CHeader::GetVersion()
{
	return Version;
}

CHeader::CHeader()
{
	Type = 0;
	Size = 0;
	Version = 0;
}

CHeader::CHeader(uint32 t, uint32 s, uint32 v)
{
	Type = t;
	Size = s;
	Version = v;
}

CHeader::~CHeader()
{
}
