#include <iostream>
#include <cstdlib>
#include <cstring>

#include "txd.h"
using namespace std;

void ReadFromFile1(istream &txd)
{
	uint32 end;

	txd.seekg(0, ios::end);
	end = txd.tellg();
	ReadFromFile(txd, 0, end);
}

void CTxd::ReadFromFile(istream &txd, uint32 start, uint32 end)
{
	uint32 i;
	uint32 platform;
	CHeader rwh;

	txd.seekg(start, ios::beg);

	rwh.Read(txd);
	if (rwh.GetType() != 0x16) {
		cerr << "Not a txd file\n";
		exit(1);
	}
	txd.seekg(12, ios::cur);
	numTextures = ReadWord(txd);
	txd.seekg(2, ios::cur);
	Tex = new SPicture[numTextures];

	for (i = 0; i < numTextures; i++) {
		txd.seekg(12, ios::cur);
		rwh.Read(txd);
		platform = ReadDword(txd);
		if (platform != TPS2 && platform != TPC) {
			cerr << "Unsupported platform\n";
			exit(2);
		}
		if (platform == TPS2)
			ReadPs2Txd(txd, Tex[i]);
		else
			ReadPcTxd(txd, Tex[i], rwh.GetSize());
		rwh.Read(txd);
		txd.seekg(rwh.GetSize(), ios::cur);
	}
}

void unswizzle8(byte *indices, byte *rawindices, uint32 Width, uint32 Height)
{
	uint32 x, y;
	for (y = 0; y < Height; y++)
		for (x = 0; x < Width; x++) {
			int block_loc = (y&(~0x0F))*Width + (x&(~0x0F))*2;
			unsigned int swap_sel = (((y+2)>>2)&0x01)*4;
			int ypos = (((y&(~3))>>1) + (y&1))&0x07;
			int column_loc = ypos*Width*2 + ((x+swap_sel)&0x07)*4;
			int byte_sum = ((y>>1)&1) + ((x>>2)&2);

			int swizzled = block_loc + column_loc + byte_sum;
			indices[y*Width+x] = rawindices[swizzled];
		}
}

void correctindices(byte *indices, uint32 Width, uint32 Height)
{
	uint32 i;
	byte tmp;
	byte map[4] = { 0, 16, 8, 24 };
	for (i = 0; i < Width*Height; i++) {
		tmp = indices[i];
		indices[i] = (tmp & ~24) | map[(tmp & 24) >> 3];
	}
}

void CTxd::ReadPcTxd(istream &txd, SPicture &tex, uint32 size)
{
	uint32 i;
	byte ind;
	uint32 end;
	uint32 picsize;
	byte pal[256*4];

	end = size - 4 + txd.tellg();

	tex.FilterFlags = ReadWord(txd);
	txd.read(tex.Name, 32);
	txd.read(tex.Alpha, 32);
	tex.RasterFormat = ReadDword(txd);

	tex.HasAlpha = ReadDword(txd);
	tex.Width = ReadDword(txd);
	tex.Height = ReadDword(txd);
	tex.Bpp = ReadByte(txd);
	tex.MipmapCount = ReadByte(txd);
	txd.seekg(1, ios::cur);
	tex.Compression = ReadByte(txd);
	tex.Size = NULL;
	picsize = tex.Width * tex.Height;

	if ((tex.RasterFormat & 0x2000) != 0)
		txd.read((char*)pal, 256*4);
	tex.Data = new byte[picsize*4];
	tex.Data_a = new byte[picsize*4];

	txd.seekg(4, ios::cur);
	if ((tex.RasterFormat & 0x2000) != 0) {
		for (i = 0; i < picsize; i++) {
			ind = ReadByte(txd);
			tex.Data[i*4] = tex.Data[i*4] = pal[ind*4];
			tex.Data[i*4+1] = tex.Data[i*4+1] = pal[ind*4+1];
			tex.Data[i*4+2] = tex.Data[i*4+2] = pal[ind*4+2];
			tex.Data[i*4+3] = pal[ind*4+3];
			tex.Data[i*4+3] = 0xFF;
		}
		tex.Bpp = 32;
	} else if (tex.Bpp == 32) {
		txd.read((char*)tex.Data_a, picsize*4);
		tex.Data = NULL;
	} else if (tex.Bpp == 16) {
		cerr << "16 bits not yet supported\n";
		exit(3);
	}

	txd.seekg(end);
}

void CTxd::ReadPs2Txd(istream &txd, SPicture &tex)
{
	uint32 i;
	uint32 picsize;
	byte ind;
	byte *rawindices, *indices, *pal;
	uint32 psize;
	CHeader rwh;

	tex.FilterFlags = ReadDword(txd);

	rwh.Read(txd);
	txd.read(tex.Name, rwh.GetSize());

	rwh.Read(txd);
	txd.read(tex.Alpha, rwh.GetSize());
	if (tex.Alpha[0] == '\0')
		tex.HasAlpha = 0;
	else
		tex.HasAlpha = 1;

	txd.seekg(24, ios::cur);
	tex.Width = ReadDword(txd);
	tex.Height = ReadDword(txd);
	tex.Bpp = ReadDword(txd);

	
#if VERBOSE
	cout << "Reading: " << tex.Name << " " << tex.Width;
	cout << " " << tex.Height << " " << tex.Bpp << endl;
#endif

	txd.seekg(144, ios::cur);

	/* Pixels/Indices */
	picsize = tex.Width * tex.Height;
	if (tex.Bpp == 8) {
		rawindices = new byte[picsize];
		indices = new byte[picsize];
		txd.read((char*)rawindices, picsize);
		unswizzle8(indices, rawindices, tex.Width, tex.Height);
		delete[] rawindices;
		correctindices(indices, tex.Width, tex.Height);
	} else if (tex.Bpp == 4) {
		rawindices = new byte[picsize];
		indices = new byte[picsize];
		for (i = 0; i < picsize; i += 2) {
			ind = ReadByte(txd);
			rawindices[i+1] = ind >> 4;
			rawindices[i] = ind & 0x0F;
		}
		if (rwh.GetVersion() != 0x310)
			unswizzle8(indices, rawindices, tex.Width, tex.Height);
		else
			memcpy(indices, rawindices, picsize);
		delete[] rawindices;
	} else if (tex.Bpp == 16) {
		tex.Data = new byte[picsize*4];
		tex.Data_a = new byte[picsize*4];
		for (i = 0; i < picsize*4; i += 4) {
			uint16 tmp;
			tmp = ReadWord(txd);
			tex.Data[i] = tex.Data_a[i] = (tmp & 0x1F) * 8;
			tmp >>= 5;
			tex.Data[i+1] = tex.Data_a[i+1] = (tmp & 0x1F) * 8;
			tmp >>= 5;
			tex.Data[i+2] = tex.Data_a[i+2] = (tmp & 0x1F) * 8;
			tmp >>= 5;
			tex.Data_a[i+3] = (tmp & 0x01) * 8;
			if (tex.Data_a[i+3] != 0xFF)
				tex.HasAlpha = 1;
			tex.Data[i+3] = 0xFF;
		}
		tex.Bpp = 32;
	} else if (tex.Bpp == 32) {
		tex.Data = new byte[picsize*4];
		tex.Data_a = new byte[picsize*4];
		for (i = 0; i < picsize*4; i += 4) {
			tex.Data[i] = tex.Data_a[i] = ReadByte(txd);
			tex.Data[i+1] = tex.Data_a[i+1] = ReadByte(txd);
			tex.Data[i+2] = tex.Data_a[i+2] = ReadByte(txd);
			tex.Data_a[i+3] = ReadByte(txd);
			if (tex.Data_a[i+3] != 0) {
				tex.Data_a[i+3] *= 2;
				tex.Data_a[i+3]-- ;
			}
			if (tex.Data_a[i+3] != 0xFF)
				tex.HasAlpha = 1;
			tex.Data[i+3] = 0xFF;
		}
	} else {
		cerr << "Unknown Depth: " << tex.Bpp << endl;
		cerr << tex.Name << endl;
		exit(4);
	}


	/* Read palette */
	if (tex.Bpp == 8 || tex.Bpp == 4) {
		txd.seekg(80, ios::cur);

		/* Don't know what's happening there */
		if (tex.Bpp == 4 && tex.Width == 16)
			txd.seekg(132, ios::cur);

		if (tex.Bpp == 8)
			psize = 256;
		else
			psize = 16;
		pal = new byte[psize*4];
		txd.read((char*)pal, psize*4);
	}
	if (rwh.GetVersion() != 0x310 && tex.Bpp == 4)
		txd.seekg(32, ios::cur);
	/* See above */
	if (rwh.GetVersion() != 0x310 && tex.Bpp == 4 && tex.Width == 16)
		txd.seekg(-4, ios::cur);

	/* Convert to 32 bit picture */
	if (tex.Bpp == 4 || tex.Bpp == 8) {
		tex.Data = new byte[picsize*4];
		tex.Data_a = new byte[picsize*4];
		for (i = 0; i < picsize; i++) {
			tex.Data[i*4] = tex.Data_a[i*4] = pal[indices[i]*4];
			tex.Data[i*4+1] = tex.Data_a[i*4+1]=pal[indices[i]*4+1];
			tex.Data[i*4+2] = tex.Data_a[i*4+2]=pal[indices[i]*4+2];
			tex.Data_a[i*4+3] = pal[indices[i]*4+3];
			if (tex.Data_a[i*4+3] != 0) {
				tex.Data_a[i*4+3] *= 2;
				tex.Data_a[i*4+3]-- ;
			}
			if (tex.Data_a[i*4+3] != 0xFF)
				tex.HasAlpha = 1;
			tex.Data[i*4+3] = 0xFF;
		}
		tex.Bpp = 32;
		delete[] pal;
		delete[] indices;
	}
	tex.RasterFormat = 0;
	tex.MipmapCount = 0;
	tex.Compression = 0;
	tex.Size = NULL;
}

CTxd::CTxd()
{
	numTextures = 0;
	Tex = NULL;
}

CTxd::~CTxd()
{
	uint32 i;

	if (Tex == NULL)
		return;

	for (i = 0; i < numTextures; i++) {
		delete[] Tex[i].Data;
		delete[] Tex[i].Data_a;
		delete[] Tex[i].Size;
	}
}
