#include <iostream>
#include <cstring>
#include "mapviewer.h"
using namespace std;

char *GetWordUp(char *line, char *word)
{
	uint i;

	while (*line == ' ' || *line == '\t' || *line == ',')
		line++;
	sscanf(line, "%s", word);
	while (*line != ' ' && *line != '\t' && *line != ',')
		line++;

	for (i = 0; i < strlen(word); i++) {
		if (word[i] == ',')
			word[i] = '\0';
		if (islower(word[i]))
			word[i] = toupper(word[i]);
	}
	return line;
}

char *GetFloat(char *line, float *number)
{
	char buffer[24];

	line = GetWordUp(line, buffer);
	*number = atof(buffer);
	return line;
}

char *GetInt(char *line, void *number)
{
	char buffer[24];
	int *p;

	p = (int *) number;
	line = GetWordUp(line, buffer);
	*p = atoi(buffer);
	return line;
}

char *GetRGB(char *line, byte *color)
{
	char buffer[24];

	line = GetWordUp(line, buffer);
	color[0] = (byte) atoi(buffer);
	line = GetWordUp(line, buffer);
	color[1] = (byte) atoi(buffer);
	line = GetWordUp(line, buffer);
	color[2] = (byte) atoi(buffer);
	color[3] = 255;
	return line;
}

char *GetRGBA(char *line, byte *color)
{
	char buffer[24];

	line = GetWordUp(line, buffer);
	color[0] = (byte) atoi(buffer);
	line = GetWordUp(line, buffer);
	color[1] = (byte) atoi(buffer);
	line = GetWordUp(line, buffer);
	color[2] = (byte) atoi(buffer);
	line = GetWordUp(line, buffer);
	color[3] = (byte) atoi(buffer);
	return line;
}

char *ReadConfigLine(FILE *file, char *buffer)
{
	uint i;
	char Line[512];

readline:
	if (fgets(Line, 512, file) == NULL)
		return NULL;
	while (1) {
		if (Line[strlen(Line)-1] == 0xd || Line[strlen(Line)-1] == 0xa)
			Line[strlen(Line)-1] = '\0';
		else
			break;
	}
	for (i = 0; i < strlen(Line); i++)
		if (Line[i] == '#' || Line[i] == ';' ||
		    (Line[i] == '/' && Line[i+1] == '/')) {
			Line[i] = '\0';
			break;
		}
	if (Line[0] == '\0')
		goto readline;

	strcpy(buffer, Line);
	return buffer;
}

RpClump *CFileLoader::LoadClump(DirEntry *Entry)
{
	RwStream *fdff;
	RpClump *Clump;

	fdff = RwStreamOpen(rwSTREAMFILENAME, rwSTREAMREAD, Entry->ContFile);
	if (fdff == NULL) {
		cerr << "Couldn't open " << Entry->ContFile << endl;
		exit(11);
	}
	Clump = RpClumpStreamRead2(fdff, Entry->Start,Entry->Start+Entry->Size);
	RwStreamClose(fdff, NULL);

	return Clump;
}

RwTexDictionary *CFileLoader::LoadTexDict(DirEntry *Entry)
{
	RwStream *ftxd;
	RwTexDictionary *Dict;

	ftxd = RwStreamOpen(rwSTREAMFILENAME, rwSTREAMREAD, Entry->ContFile);
	if (ftxd == NULL) {
		cerr << "Couldn't open " << Entry->ContFile << endl;
		exit(12);
	}
	Dict = RwTexDictionaryStreamRead2(ftxd, Entry->Start);
	RwStreamClose(ftxd, NULL);
	LoadGLTextures(Dict);
	FreeRwTextures(Dict);

	return Dict;
}

bool CFileLoader::LoadDff(CWorldObj *obj)
{
	DirEntry *Entry;
	RpClump *Clump;

	if ((Entry = obj->DffEntry) == NULL)
		return false;
	if (Entry->Unloadable)
		return false;
	if (Entry->Data == NULL) {
		cout << Entry->FileName << " not loaded...loading\n";
		Clump = LoadClump(Entry);
		if (Clump == NULL) {
			Entry->Unloadable = true;
			return false;
		}
		Entry->Data = Clump;
		obj->SetClump(Clump);
	} else {
		obj->SetClump((RpClump *) Entry->Data);
	}
	return true;
}

bool CFileLoader::LoadTxd(CWorldObj *obj)
{
	DirEntry *Entry, *ParEntry;
	RwTexDictionary *Dict, *ParDict;
	
	if ((Entry = obj->TxdEntry) == NULL)
		return false;
	if (Entry->Data == NULL) {
		cout << Entry->FileName << " not loaded...loading\n";
		Dict = LoadTexDict(Entry);
		Entry->Data = Dict;
		obj->TexDict = Dict;
		Dict->Parent = NULL;
		ParEntry = Entry->Parent;
		if (ParEntry != NULL) {
			if (ParEntry->Data == NULL) {
				cout << ParEntry->FileName <<
				  " not loaded...loading\n";
				ParDict = LoadTexDict(ParEntry);
				ParEntry->Data = ParDict;
				Dict->Parent = ParDict;
				ParDict->Parent = NULL;
			} else {
				Dict->Parent = (RwTexDictionary *)
				  ParEntry->Data;
			}
		}
	} else {
		obj->TexDict = (RwTexDictionary *) Entry->Data;
	}
	return true;
}

void CFileLoader::UnloadWorldObj(CWorldObj *obj)
{
}

void CFileLoader::FreeRwTextures(RwTexDictionary *TexDict)
{
	RwUInt32 i;

	for (i = 0; i < TexDict->numTextures; i++) {
		delete[] TexDict->texture[i].Data;
		delete[] TexDict->texture[i].Data_a;
		TexDict->texture[i].Data = NULL;
		TexDict->texture[i].Data_a = NULL;
	}
}

void CFileLoader::LoadGLTextures(RwTexDictionary *TexDict)
{
	RwUInt32 i;
	GLuint *texid;

	texid = new GLuint[TexDict->numTextures];
	glGenTextures(TexDict->numTextures, texid);
	for (i = 0; i < TexDict->numTextures; i++) {
		glBindTexture(GL_TEXTURE_2D, texid[i]);

		glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);

		if (TexDict->texture[i].depth == 32) {
			if (TexDict->texture[i].reversed)
				glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA,
					     TexDict->texture[i].width,
					     TexDict->texture[i].height,
					     0, GL_BGRA, GL_UNSIGNED_BYTE,
					     TexDict->texture[i].Data_a);
			else
				glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA,
					     TexDict->texture[i].width,
					     TexDict->texture[i].height,
					     0, GL_RGBA, GL_UNSIGNED_BYTE,
					     TexDict->texture[i].Data_a);
		} else if (TexDict->texture[i].depth == 16) {
			cout << "Found 16 bit picture\n";
			glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA,
			             TexDict->texture[i].width,
			             TexDict->texture[i].height,
			             0, GL_RGBA, GL_UNSIGNED_SHORT_5_5_5_1,
			             TexDict->texture[i].Data_a);
		} else {
			cerr << "Can't handle non 32bit textures " <<
			  TexDict->texture[i].depth << i << "\n";
			exit(1);
		}
		TexDict->texture[i].TexID = texid[i];
	}
}

void CFileLoader::LoadBinIpl(DirEntry *Entry)
{
	FILE *ipl;

	if ((ipl = fopen(Entry->ContFile, "rb")) == NULL) {
		cerr << "Couldn't open file: " << Entry->ContFile << endl;
		exit(1);
	}
	fseek(ipl, Entry->Start, SEEK_SET);
	InstanceList.AppendFromBinFile(ipl);
	fclose(ipl);
}
