#ifdef WIN32
  #include <windows.h>
#endif
#ifdef GLEW
  #include <GL/glew.h>
#endif
#include <iostream>

#include <time.h>

#include "mapviewer.h"

#ifdef GLX
  #include "glx.h"
#else
  #include "glut.h"
#endif

using namespace std;

GLfloat lgtdif[] = { 0.8f, 0.8f, 0.8f, 1.0f };
GLfloat lgtamb[] = { 1.0f, 1.0f, 1.0f, 1.0f };
GLfloat lgtspe[] = { 1.0f, 1.0f, 1.0f, 1.0f };
GLfloat lgtpos[] = { 0.0f, 0.0f, 2.0f, 1.0f };

void TransformItem(Instance *Inst)
{
	RwReal *quat;
	RwReal x2, y2, z2, xy, xz, yz, wx, wy, wz;
	GLfloat matrix[16];

	quat = Inst->Rotation;

	x2 = quat[0] * quat[0];
	y2 = quat[1] * quat[1];
	z2 = quat[2] * quat[2];
	xy = quat[0] * quat[1];
	xz = quat[0] * quat[2];
	yz = quat[1] * quat[2];
	wx = quat[3] * quat[0];
	wy = quat[3] * quat[1];
	wz = quat[3] * quat[2];
	matrix[0] = 1.0f - 2.0f * (y2 + z2);
	matrix[1] = 2.0f * (xy - wz);
	matrix[2] = 2.0f * (xz + wy);
	matrix[3] = 0.0f;
	matrix[4] = 2.0f * (xy + wz);
	matrix[5] = 1.0f - 2.0f * (x2 + z2);
	matrix[6] = 2.0f * (yz - wx);
	matrix[7] = 0.0f;
	matrix[8] = 2.0f * (xz - wy);
	matrix[9] = 2.0f * (yz + wx);
	matrix[10] = 1.0f - 2.0f * (x2 + y2);
	matrix[11] = 0.0f;
	matrix[12] = Inst->PositionV[0];
	matrix[13] = Inst->PositionV[1];
	matrix[14] = Inst->PositionV[2];
	matrix[15] = 1.0f;

	glMultMatrixf(matrix);
}

bool isVisible(CWorldObj *curobj)
{
	int TimeHour;

	TimeHour = Clock.GetHour();
	if (curobj->TimeOn < curobj->TimeOff) { /* Day object */
		if (TimeHour < curobj->TimeOn ||
		    TimeHour >= curobj->TimeOff)
			return false;
		else
			return true;
	} else {                                /* Night object */
		if (TimeHour < curobj->TimeOff ||
		    TimeHour >= curobj->TimeOn)
			return true;
		else
			return false;
	}
}

void DrawWorld(int DrawTransp)
{
	int i;
	int numInstances;
	Instance *curinst;

	numInstances = InstanceList.GetLength();

	for (i = 0; i < numInstances; i++) {
		curinst = InstanceList.GetInst(i);

		/* These Don't work with GTA3 PS2, don't know why */
		if (strcmp(curinst->Name, "TRAFFICCONE") == 0)
			continue;
		if (strcmp(curinst->Name, "PARKTABLE1") == 0)
			continue;

		if ((Game & GAMEVICECITY) != 0)
			if (Interior != curinst->Interior &&
			    curinst->Interior != 13)
				continue;

//#if 0
		/* LODs that have no high-resolution Model */
		if (curinst->isLOD && curinst->Parent != NULL)
			continue;

		if (curinst->isLOD && curinst->Parent == NULL &&
		    Camera.DistanceTo(curinst->PositionV) < 60)
			continue;

		curinst = InstanceList.GetCorrectInst(curinst);
		if (curinst == NULL)
			continue;

		if (curinst->Obj->isTimed)
			if (!isVisible(curinst->Obj))
				continue;
//#endif

		/* Load DFF, if that hasn't been done */
		if (curinst->Obj->Clump == NULL)
			if (!FileLoader.LoadDff(curinst->Obj))
				continue;

		if (UseTextures) {
			/* Load TXD, if that hasn't been done */
			if (curinst->Obj->TexDict == NULL)
				if (!FileLoader.LoadTxd(curinst->Obj))
					continue;
		}

		/* Draw it */
		RwTexDictionarySetCurrent(curinst->Obj->TexDict);
		glPushMatrix();
		TransformItem(curinst);
		curinst->Obj->Draw(0, DrawTransp);
		glPopMatrix();
	}
}

void DrawRadar()
{
	int size, bottom, top, left, right;
	RwTexDictionary *hud;
	RwTexture *tex;
	DirEntry *Entry;

	if (!UseTextures)
		return;

	size = getHeight() / 4;
	bottom = getHeight() - size/3;
	top = bottom - size;
	left = size/3;
	right = left + size;

	Entry = Directory.GetEntry("HUD", ".TXD");
	if (Entry == NULL)
		return;
	if (Entry->Data == NULL) {
		hud = FileLoader.LoadTexDict(Entry);
		Entry->Data = hud;
	} else {
		hud = (RwTexDictionary *) Entry->Data;
	}
	tex = RwTexDictionaryFindNamedTexture(hud, "radardisc");
	if (tex != NULL) {
		glBindTexture(GL_TEXTURE_2D, tex->TexID);
	} else {
		glBindTexture(GL_TEXTURE_2D, 0);
	}
	glColor3ub(255, 255, 255);
	glBegin(GL_POLYGON);
	glMultiTexCoord2f(GL_TEXTURE0, 0.0f, 0.0f);
	glVertex2f(left, top);
	glMultiTexCoord2f(GL_TEXTURE0, 1.0f, 0.0f);
	glVertex2f(right, top);
	glMultiTexCoord2f(GL_TEXTURE0, 1.0f, 1.0f);
	glVertex2f(right, bottom);
	glMultiTexCoord2f(GL_TEXTURE0, 0.0f, 1.0f);
	glVertex2f(left, bottom);
	glEnd();

}

void RenderScene()
{
	int w, h;
	static int Frame = 0;;
	static time_t StartTime = time(NULL);
	color tintcol;

	w = getWidth();
	h = getHeight();

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();

	Camera.Look();

	glBindTexture(GL_TEXTURE_2D, 0);
	glColor4ub(255, 255, 255, 255);

	Sky.Draw();
	DrawWorld(0);
	if ((Game & GAMESANANDREAS) == 0)
		Water.Draw();
	DrawWorld(1);

	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();
	glOrtho(0, w, h, 0, -1, 1);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glDisable(GL_DEPTH_TEST);

	if ((Game & GAMEIII) != 0 && UseTint) {
		Sky.GetTintColor(tintcol);
		glBindTexture(GL_TEXTURE_2D, 0);
		glEnable(GL_BLEND);
		glColor4ubv(tintcol);
		glBegin(GL_POLYGON);
		glVertex2f(0, h);
		glVertex2f(w, h);
		glVertex2f(w, 0);
		glVertex2f(0, 0);
		glEnd();
	}
/*
	if ((Game & GAMESANANDREAS) == 0)
		DrawRadar();
*/

	glEnable(GL_DEPTH_TEST);
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);

	SwapBuffers();
	Frame++;
	if (Frame == 100) {
		cout << "100 Frames in " <<time(NULL)-StartTime << " seconds\n";
		Frame = 0;
		StartTime = time(NULL);
	}
}

void MixColors(RwUInt8 *result, RwUInt8 *col1, RwUInt8 *col2,
                          int ratio1, int full)
{
        int ratio2;

        ratio2 = full-ratio1;
        result[0] = (col1[0]*ratio1 + col2[0]*ratio2)/full;
        result[1] = (col1[1]*ratio1 + col2[1]*ratio2)/full;
        result[2] = (col1[2]*ratio1 + col2[2]*ratio2)/full;
        result[3] = (col1[3]*ratio1 + col2[3]*ratio2)/full;
}


bool BindTexture(const char *texname)
{
	bool retval;
	RwTexDictionary *Dict;
	RwTexture *rastertex;

	Dict = RwTexDictionaryGetCurrent();
	if (Dict == NULL) {
		glBindTexture(GL_TEXTURE_2D, 0);
		return false;
	}

	rastertex = RwTexDictionaryFindNamedTexture(Dict, texname);
	if (rastertex != NULL) {
		glBindTexture(GL_TEXTURE_2D, rastertex->TexID);
		return rastertex->HasAlpha;
	} else {
		RwTexDictionarySetCurrent(Dict->Parent);
		retval = BindTexture(texname);
		RwTexDictionarySetCurrent(Dict);
		return retval;
	}
}

void Init()
{
	int w, h;

	w = getWidth();
	h = getHeight();

	FOV = 45.0f;

#ifdef GLEW
	{
		GLenum err;
		err = glewInit();
		if (err != GLEW_OK) {
			cerr << "Error with GLEW\n";
			exit(1);
		}
	}
#endif
//	glClearColor(0.4f, 0.8f, 1.0f, 1.0f);
	glClearDepth(1.0);
	glDepthFunc(GL_LEQUAL);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_ALPHA_TEST);
	glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
	glShadeModel(GL_SMOOTH);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(FOV, (GLfloat) w/(GLfloat)h, 0.1f, BACKPLANE);

	glMatrixMode(GL_MODELVIEW);

	/* Set up lighting */
	glEnable(GL_LIGHTING);
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lgtamb);
	glEnable(GL_COLOR_MATERIAL);
	glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE);

	/* Set up texture units */
	glActiveTexture(GL_TEXTURE0);
	glEnable(GL_TEXTURE_2D);
	glActiveTexture(GL_TEXTURE1);
	glEnable(GL_TEXTURE_2D);

	/* Set up texture generation for reflection maps */
	glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
	glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);

	glEnable(GL_TEXTURE_GEN_S);
	glEnable(GL_TEXTURE_GEN_T);
	glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_ADD);
	glActiveTexture(GL_TEXTURE0);

	Camera.SetYaw(1.4387);
	Camera.SetPitch(-0.5176);
	Camera.SetPosition(852.0, -1120.0, 260.0);
	Camera.ReadSavedPositions();

	if ((Game & GAMESANANDREAS) == 0)
		Water.ReadWaterData();
	Sky.ReadWeatherData();

	Clock.SetHour(12);
	Clock.SetMinute(0);

	RwTexDictionarySetCurrent(NULL);

	Interior = 0;
}

void DeInit()
{
}
