#ifdef WIN32
  #include <windows.h>
#endif

#include <iostream>
#include <cstdio>
#include <fstream>
#include <cstring>

#include <unistd.h>

#include <rwcore.h>
#include <rpworld.h>

#include "mapviewer.h"

using namespace std;

uint Game;
char progname[30];
bool UseTextures;
bool UseVertexColors;
bool UseFog;
bool UseTint;
uint Interior;
char *camposfile;
GLfloat FOV;

CCamera Camera;
CDirectory Directory;
CObjectList ObjectList;
CInstanceList InstanceList;
CFileLoader FileLoader;
CWater Water;
CSky Sky;
CClock Clock;

GLfloat planex = 0;
GLfloat planey = 0;
GLfloat planescale = 1;

void CorrectPath(char *Path)
{
	while (*Path != '\0') {
		if (*Path == '\\')
			*Path = '/';
		Path++;
	}
}

char *GetFileName(char *Path)
{
	return strrchr(Path, '/') + 1;
}

void ParseDat(FILE *conf)
{
	FILE *file;
	char Buffer[512];
	char Type[24];
	char Path[128];
	char *Pos;
	char *FileName;

	while (ReadConfigLine(conf, Buffer) != NULL) {
		Pos = Buffer;
		Pos = GetWordUp(Pos, Type);
		if (strcmp(Type, "IDE") == 0) {
			Pos = GetWordUp(Pos, Path);
			CorrectPath(Path);
			cout << Path << endl;
			if ((file = fopen(Path, "r")) == NULL) {
				cerr << "Couldn't open file\n";
				exit(1);
			}
			ObjectList.AppendFromFile(file);
			fclose(file);
		} else if (strcmp(Type, "IPL") == 0) {
			Pos = GetWordUp(Pos, Path);
			CorrectPath(Path);
			FileName = GetFileName(Path);
			cout << Path << endl;
			if ((file = fopen(Path, "r")) == NULL) {
				cerr << "Couldn't open file\n";
				exit(1);
			}
			InstanceList.AppendFromFile(file, FileName);
			fclose(file);
		} else if (strcmp(Type, "TEXDICTION") == 0) {
			Pos = GetWordUp(Pos, Path);
			CorrectPath(Path);
			FileName = GetFileName(Path);
			cout << Path << endl;
			if ((file = fopen(Path, "r")) == NULL) {
				cerr << "Couldn't open file\n";
				exit(1);
			}
			Directory.Append(FileName, 0, 0, Path, NULL);
			fclose(file);
		} else if (strcmp(Type, "MODELFILE") == 0) {
			Pos = GetWordUp(Pos, Path);
			CorrectPath(Path);
			FileName = GetFileName(Path);
			cout << Path << endl;
			if ((file = fopen(Path, "r")) == NULL) {
				cerr << "Couldn't open file\n";
				exit(1);
			}
			Directory.Append(FileName, 0, 0, Path, NULL);
			fclose(file);
		} else if (strcmp(Type, "IMG") == 0) {
			Pos = GetWordUp(Pos, Path);
			CorrectPath(Path);
			cout << Path << endl;
			if ((file = fopen(Path, "r")) == NULL) {
				cerr << "Couldn't open file\n";
				exit(1);
			}
			Directory.AppendFromDir(file, Path);
			fclose(file);
		}
	}
}

void BuildTree()
{
	int i;
	int base;
	int numInstances;
	Instance *curinst;
	Instance *child;

	numInstances = InstanceList.GetLength();

	cout << "Loading, please wait..." << flush;
	for (i = 0; i < numInstances; i++) {
		curinst = InstanceList.GetInst(i);

		/* Give every instance its object */
		if (curinst->Obj == NULL) {
			curinst->Obj = ObjectList.GetObj(curinst->ID);
			if (curinst->Obj == NULL) {
				cout << "Didn't find obj with id: " <<
				  curinst->ID << endl;
				continue;
			}
			curinst->Obj->RefCount++;
		}

		/* Link LODs and non-LODs */
		if ((Game & GAMESANANDREAS) == 0) {
			if (!curinst->isLOD && curinst->Child == NULL) {
				curinst->Child = InstanceList.GetLOD(curinst);
				if (curinst->Child != NULL)
					curinst->Child->Parent = curinst;
			}
		} else {
			/* When Index == 0, we're at the beginning of a file */ 
			if (curinst->Index == 0)
				base = i;
			if (curinst->LOD != -1) {
				child = InstanceList.GetInst(base+curinst->LOD);
				curinst->Child = child;
				child->Parent = curinst;
			}
		}

		/* Give every object its dff and txd */
		if (curinst->Obj->Clump == NULL) {
			curinst->Obj->DffEntry =
			  Directory.GetEntry(curinst->Obj->ModelName, ".DFF");
		}
		if (curinst->Obj->TexDict == NULL) {
			curinst->Obj->TxdEntry =
			  Directory.GetEntry(curinst->Obj->TextureName, ".TXD");
		}
	}
	cout << "done\n";
}

int main(int argc, char *argv[])
{
	FILE *file, *conf;
	char Path[512];

	UseTextures = true;
	UseVertexColors = true;
	UseFog = false;
	UseTint = true;

	Game = 0;
	strcpy(progname, "Map Viewer");

	if ((file = fopen("mapviewer.conf", "r")) == NULL) {
		cerr << "Couldn't open conf file\n";
		return 1;
	}
	ReadConfigLine(file, Path);
	fclose(file);

	camposfile = NULL;
	camposfile = getwd(camposfile);
	camposfile = (char *) realloc(camposfile,
	                     strlen(camposfile)+strlen("/campositions.conf"));
	strcat(camposfile, "/campositions.conf");

	chdir(Path);
	
	if ((conf = fopen("DATA/GTA3.DAT", "r")) != NULL) {
		Game |= GAMEIII;
	} else if ((conf = fopen("DATA/GTA_VC.DAT", "r")) != NULL) {
		Game |= GAMEVICECITY;
	} else if ((conf = fopen("DATA/GTA.DAT", "r")) != NULL) {
		Game |= GAMESANANDREAS;
	} else {
		cerr << "Couldn't open dat file\n";
		return 1;
	}

	if ((Game & GAMESANANDREAS) != 0) {
		if ((file = fopen("MODELS/GTA3.IMG", "r")) == NULL) {
			cerr << "Couldn't open gta3.img\n";
			return 1;
		}
		Directory.AppendFromDir(file, "MODELS/GTA3.IMG");
		fclose(file);

		if ((file = fopen("MODELS/GTA_INT.IMG", "r")) == NULL) {
			cerr << "Couldn't open gta_int.img\n";
			return 1;
		}
		Directory.AppendFromDir(file, "MODELS/GTA_INT.IMG");
		fclose(file);

		Directory.Append("EFFECTS.TXD",0,0,"MODELS/EFFECTS.TXD", NULL);
		Directory.Append("PARTICLE.TXD",0,0,"MODELS/PARTICLE.TXD",NULL);
		Directory.Append("HUD.TXD",0,0,"MODELS/HUD.TXD", NULL);
		UseFog = true;
	} else if ((Game & GAMEVICECITY) != 0) {
		if ((file = fopen("MODELS/GTA3.DIR", "r")) == NULL) {
			cerr << "Couldn't open dir file\n";
			return 1;
		}
		Directory.AppendFromDir(file, "MODELS/GTA3.IMG");
		fclose(file);

		Directory.Append("GENERIC.TXD",0,0,"MODELS/GENERIC.TXD", NULL);
		Directory.Append("PARTICLE.TXD",0,0,"MODELS/PARTICLE.TXD",NULL);
		Directory.Append("HUD.TXD",0,0,"MODELS/HUD.TXD", NULL);
		Directory.Append("AAA.TXD",0,0,"MODELS/AAA.TXD", NULL);
	} else if ((Game & GAMEIII) != 0) {
		if ((file = fopen("MODELS/GTA3.DIR", "r")) == NULL) {
			cerr << "Couldn't open dir file\n";
			return 1;
		}
		Directory.AppendFromDir(file, "MODELS/GTA3.IMG");
		fclose(file);

		Directory.Append("MISC.TXD",0,0,"MODELS/MISC.TXD", NULL);
		Directory.Append("PARTICLE.TXD",0,0,"MODELS/PARTICLE.TXD",NULL);
		Directory.Append("HUD.TXD",0,0,"MODELS/HUD.TXD", NULL);
	}

	ParseDat(conf);
	fclose(conf);

	BuildTree();
	StartOgl(argc, argv);

	return 0;
}
