#include <iostream>
#include <vector>

#include "mapviewer.h"

using namespace std;

enum BlockTypeAll {
	END = 0
};

enum BlockTypeIpl {
	INST = 1,
	CULL
};

enum BlockTypeIde {
	OBJS = 1,
	TOBJ,
	PATH,
	TWODFX,
	ANIM,
	TXDP
};

void CInstanceList::AppendFromBinFile(FILE *file)
{
	int i;
	int numEntries;
	Instance Inst;
	CWorldObj *Obj;

	fseek(file, 4, SEEK_CUR);
	fread(&numEntries, 4, 1, file);
	fseek(file, 68, SEEK_CUR);
	for (i = 0; i < numEntries; i++) {
		fread(Inst.PositionV, 4, 3, file);
		fread(Inst.Rotation, 4, 4, file);
		fread(&Inst.ID, 4, 1, file);
		fread(&Inst.Interior, 4, 1, file);
		fread(&Inst.LOD, 4, 1, file);
		Inst.Obj = NULL;
		Inst.Child = NULL;
		Inst.Parent = NULL;
		Inst.Index = Index;
		Obj = ObjectList.GetObj(Inst.ID);
		if (Obj != NULL)
			strcpy(Inst.Name, Obj->ModelName);
		InstList.push_back(Inst);
		Index++;
	}
}

void CInstanceList::AppendFromFile(FILE *file, char *Name)
{
	int i;
	DirEntry *Entry;
	char BinName[32];
	char EntryName[48];

	Index = 0;
	AppendFromPlainFile(file);

	strcpy(BinName, Name);
	BinName[strlen(BinName)-4]= '\0';
	strcat(BinName, "_STREAM");

	i = 0;
	sprintf(EntryName, "%s0.IPL", BinName);
	while ((Entry = Directory.GetEntry(EntryName)) != NULL) {
		FileLoader.LoadBinIpl(Entry);
		sprintf(EntryName, "%s%d.IPL", BinName, i++);
	}
}

void CInstanceList::AppendFromPlainFile(FILE *file)
{
	int BlockType;
	char buffer[512];
	char *pos;
	Instance Inst;

	BlockType = 0;
	while (ReadConfigLine(file, buffer) != NULL) {
		if (strcmp(buffer, "inst") == 0) {
			BlockType = INST;
			continue;
		}
		if (strcmp(buffer, "cull") == 0) {
			BlockType = CULL;
			continue;
		}
		if (strcmp(buffer, "end") == 0) {
			BlockType = END;
			continue;
		}
		if (BlockType == INST) {
			pos = buffer;

			Inst.Index = Index;
			pos = GetInt(pos, &Inst.ID);
			pos = GetWordUp(pos, Inst.Name);
			if ((Game & GAMEIII) == 0)
				pos = GetInt(pos, &Inst.Interior);
			pos = GetFloat(pos, &Inst.PositionV[0]);
			pos = GetFloat(pos, &Inst.PositionV[1]);
			pos = GetFloat(pos, &Inst.PositionV[2]);
			if ((Game & GAMESANANDREAS) == 0) {
				pos = GetFloat(pos, &Inst.ScaleV[0]);
				pos = GetFloat(pos, &Inst.ScaleV[1]);
				pos = GetFloat(pos, &Inst.ScaleV[2]);
			}
			pos = GetFloat(pos, &Inst.Rotation[0]);
			pos = GetFloat(pos, &Inst.Rotation[1]);
			pos = GetFloat(pos, &Inst.Rotation[2]);
			pos = GetFloat(pos, &Inst.Rotation[3]);
			if ((Game & GAMESANANDREAS) != 0)
				pos = GetInt(pos, &Inst.LOD);
			else
				Inst.LOD = -1;
			Inst.Obj = NULL;

			if (Inst.Name[0] == 'L' &&
			    Inst.Name[1] == 'O' &&
			    Inst.Name[2] == 'D')
				Inst.isLOD = true;
			else
				Inst.isLOD = false;
			Inst.Child = NULL;
			Inst.Parent = NULL;
			InstList.push_back(Inst);
			Index++;
		}
	}
}

void CInstanceList::Dump()
{	uint i;

	for (i = 0; i < InstList.size(); i++)
		cout << InstList[i].Name << " " << InstList[i].LOD << " "
		  << InstList[i].Index << endl;
}

Instance *CInstanceList::GetFirst()
{
	return &InstList[0];
}
	
int CInstanceList::GetLength()
{
	return InstList.size();
}

Instance *CInstanceList::GetInst(int index)
{
	return &InstList[index];
}

Instance *CInstanceList::GetLOD(Instance *inst)
{
	uint i;
	char LODName[64];

	strcpy(LODName, inst->Name);
	LODName[0] = 'L';
	LODName[1] = 'O';
	LODName[2] = 'D';

	for (i = 0; i < InstList.size(); i++) {
		if (strcmp(InstList[i].Name, LODName) == 0) {
			return &InstList[i];
		}
	}
	return NULL;
}

Instance *CInstanceList::GetCorrectInst(Instance *curinst)
{
tryagain:
	/* Check for draw distance and correct time */
	if (Camera.DistanceTo(curinst->PositionV) >
	    curinst->Obj->DrawDist[curinst->Obj->ObjCount-1]) {
		if (curinst->Child == NULL) {
			return NULL;
		} else {
			curinst = curinst->Child;
			goto tryagain;
		}
	}
	return curinst;
}

void CObjectList::AppendFromFile(FILE *file)
{
	char buffer[512];
	char TexName[48];
	char *pos;
	int BlockType;
	uint i;
	CWorldObj Obj;
	DirEntry *Entry;

	BlockType = 0;
	while (ReadConfigLine(file, buffer) != NULL) {
		if (strcmp(buffer, "objs") == 0) {
			BlockType = OBJS;
			continue;
		}
		if (strcmp(buffer, "tobj") == 0) {
			BlockType = TOBJ;
			continue;
		}
		if (strcmp(buffer, "path") == 0) {
			BlockType = PATH;
			continue;
		}
		if (strcmp(buffer, "2dfx") == 0) {
			BlockType = TWODFX;
			continue;
		}
		if (strcmp(buffer, "anim") == 0) {
			BlockType = ANIM;
			continue;
		}
		if (strcmp(buffer, "txdp") == 0) {
			BlockType = TXDP;
			continue;
		}
		if (strcmp(buffer, "end") == 0) {
			BlockType = END;
			continue;
		}
		if (BlockType == OBJS || BlockType == TOBJ ||
		    BlockType == ANIM) {
			pos = buffer;

			pos = GetInt(pos, &Obj.ID);
			pos = GetWordUp(pos, Obj.ModelName);
			pos = GetWordUp(pos, Obj.TextureName);
			if (BlockType == ANIM)
				pos = GetWordUp(pos, Obj.AnimName);
			if ((Game & GAMESANANDREAS) == 0)
				pos = GetInt(pos, &Obj.ObjCount);
			else
				Obj.ObjCount = 1;
			Obj.DrawDist = new float[Obj.ObjCount];
			for (i = 0; i < Obj.ObjCount; i++) {
				pos = GetFloat(pos, &Obj.DrawDist[i]);
			}
			pos = GetInt(pos, &Obj.Flags);

			if (BlockType == TOBJ) {
				pos = GetInt(pos, &Obj.TimeOn);
				pos = GetInt(pos, &Obj.TimeOff);
				Obj.isTimed = true;
			} else {
				Obj.isTimed = false;
			}

			Obj.Clump = NULL;
			Obj.TexDict = NULL;
			Obj.RefCount = 0;
			ObjList.push_back(Obj);
		} else if (BlockType == TXDP) {
			pos = buffer;

			pos = GetWordUp(pos, TexName);
			Entry = Directory.GetEntry(TexName, ".TXD");
			pos = GetWordUp(pos, TexName);
			if (Entry != NULL)
				Entry->Parent =
				  Directory.GetEntry(TexName, ".TXD");
		}
	}
}

CWorldObj *CObjectList::GetObj(int ID)
{
	uint i;

	for (i = 0; i < ObjList.size(); i++)
		if (ObjList[i].ID == ID)
			return &ObjList[i];
	return NULL;
}

void CObjectList::Dump()
{
	uint i;

	for (i = 0; i < ObjList.size(); i++)
		cout << ObjList[i].ID << " " << ObjList[i].ModelName << endl;
}
