#include <iostream>
#include "mapviewer.h"
using namespace std;

void CWater::DrawPatch(float x, float y, float patchsize, float height)
{
	glBegin(GL_QUADS);
	glMultiTexCoord2f(GL_TEXTURE0, 0.0f, 0.0f);
	glVertex3f(x, y, height);
	glMultiTexCoord2f(GL_TEXTURE0, 0.0f, 1.0f);
	glVertex3f(x+patchsize, y, height);
	glMultiTexCoord2f(GL_TEXTURE0, 1.0f, 1.0f);
	glVertex3f(x+patchsize, y+patchsize, height);
	glMultiTexCoord2f(GL_TEXTURE0, 1.0f, 0.0f);
	glVertex3f(x, y+patchsize, height);
	glEnd();

}

void CWater::Draw()
{
	int x, y;
	RwTexDictionary *part;
	DirEntry *Entry;
	float patchsize;
	float limits[4];
	color watercolor;

	if (!UseTextures)
		return;

	if ((Game & GAMEIII) != 0) {
		limits[0] = -2048;
		limits[1] = -2048;
		limits[2] = 2048;
		limits[3] = 2048;
	} else {
		limits[0] = -2448;
		limits[1] = -2048;
		limits[2] = 1648;
		limits[3] = 2048;
	}
	patchsize = 32;

	glBindTexture(GL_TEXTURE_2D, 0);

	Entry = Directory.GetEntry("PARTICLE", ".TXD");
	if (Entry == NULL)
		return;
	if (Entry->Data == NULL) {
		part = FileLoader.LoadTexDict(Entry);
		Entry->Data = part;
	} else {
		part = (RwTexDictionary *) Entry->Data;
	}
	RwTexDictionarySetCurrent(part);
	glColor3ub(67, 65, 100);
	if ((Game & GAMEVICECITY) != 0) {
		glEnable(GL_BLEND);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		BindTexture("waterclear256");
		glColor4ub(55, 189, 239, 128);
		Sky.GetWaterColor(watercolor);
		glColor4ubv(watercolor);
	} else if ((Game & GAMEIII) != 0) {
		BindTexture("water_old");
		glColor4ub(255, 255, 255, 255);
	}

	for (x = 0; x < 128; x++) {
		for (y = 0; y < 128; y++) {
			if (indiceshi[x*128+y] != 0x80)
				DrawPatch(x*patchsize+limits[0],
			                  y*patchsize+limits[1], patchsize,
				          palette[indiceshi[x*128+y]]);
		}
	}
}

void CWater::ReadWaterData()
{
	int i;
	byte image[128*128*4];
	byte colpal[48][4];
	FILE *wpro;

	if ((wpro = fopen("DATA/WATERPRO.DAT", "rb")) == NULL) {
		cerr << "Couldn't find waterpro.dat\n";
		exit(1);
	}
	fseek(wpro, 4, SEEK_SET);
	fread(&palette, 4, 48, wpro);
	fseek(wpro, 0x300, SEEK_CUR);
	fread(&indiceslo, 1, 64*64, wpro);
	fread(&indiceshi, 1, 128*128, wpro);
	fclose(wpro);

	memcpy(colpal, palette, 48*4);
	for (i = 0; i < 128*128; i++) {
		if (indiceshi[i] == 0x80) {
			image[i*4] = 255;
			image[i*4+1] = 255;
			image[i*4+2] = 255;
			image[i*4+3] = 255;
		} else {
			image[i*4] = colpal[indiceshi[i]][0];	
			image[i*4+1] = colpal[indiceshi[i]][1];	
			image[i*4+2] = colpal[indiceshi[i]][2];	
			image[i*4+3] = 255;
		}
	}

	glGenTextures(1, &TexID);
	glBindTexture(GL_TEXTURE_2D, TexID);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, 128, 128, 0, GL_RGBA,
	             GL_UNSIGNED_BYTE, image);
}
