#include <iostream>
#include "mapviewer.h"
using namespace std;

GLubyte carcol1[] = { 234, 205, 81, 255 };
GLubyte carcol2[] = { 80, 40, 200, 255 };

void CWorldObj::DrawGeometry(RpGeometry &g, bool DrawTransp)
{
	RwUInt32 i, j, k;
	bool isTransparent;
	RpMesh *s;
	bool hasAlpha;
	bool mixColors;
	Vector3f *vert, *norm;
	Vector2f *uv;
	Color4b *color1;
	Color4b *color2;
	RwUInt8 *meshcol;
	Color4b avcolor;

	avcolor[3] = 255;
	for (j = 0; j < g.numSplits; j++) {
		s = &g.Split[j];
		vert = g.Vertex;
		norm = g.Normal;
		uv = g.UV;

		if (g.NightVColor != NULL) {
			if (Clock.GetHour() < 21 && Clock.GetHour() >= 20) {
				mixColors = true;
				color1 = g.VColor;
				color2 = g.NightVColor;
			} else if (Clock.GetHour() >=5 && Clock.GetHour()<6) {
				mixColors = true;
				color1 = g.NightVColor;
				color2 = g.VColor;
			} else if (Clock.GetHour() >=21 || Clock.GetHour()<5) {
				mixColors = false;
				color1 = g.NightVColor;
				color2 = g.NightVColor;
			} else {
				mixColors = false;
				color1 = g.VColor;
				color2 = g.VColor;
			}
		} else {
			mixColors = false;
			color1 = g.VColor;
			color2 = g.VColor;
		}

		isTransparent = false;
		
		if (UseTextures) {
			glActiveTexture(GL_TEXTURE1);
			glBindTexture(GL_TEXTURE_2D, 0);
			glActiveTexture(GL_TEXTURE0);
			glBindTexture(GL_TEXTURE_2D, 0);

			/* if there is a diffuse map, use it */
			if (g.Mat[s->MatIndex].HasTex) {
				hasAlpha =
				  BindTexture(g.Mat[s->MatIndex].Tex.Name);
				if (hasAlpha)
					isTransparent = true;
			}

			/* if there is a reflection map, use it */
			if (g.Mat[s->MatIndex].HasRef) {
				glActiveTexture(GL_TEXTURE1);
				BindTexture(g.Mat[s->MatIndex].Reflection.Name);
				glActiveTexture(GL_TEXTURE0);
			}
		} else {
			glBindTexture(GL_TEXTURE_2D, 0);
		}

		if (g.Mat[s->MatIndex].Color[3] != 255)
			isTransparent = true;

		if (DrawTransp != isTransparent)
			continue;

		if (isTransparent) {
			glEnable(GL_BLEND);
			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
			glAlphaFunc(GL_GREATER, 0.0);
		} else {
			glDisable(GL_BLEND);
		}

		if (g.Mat[s->MatIndex].Color[0] == 60 &&
		    g.Mat[s->MatIndex].Color[1] == 255 &&
		    g.Mat[s->MatIndex].Color[2] == 0)
			meshcol = carcol1;
		else if (g.Mat[s->MatIndex].Color[0] == 255 &&
		    g.Mat[s->MatIndex].Color[1] == 0 &&
		    g.Mat[s->MatIndex].Color[2] == 175)
			meshcol = carcol2;
		else
			meshcol = g.Mat[s->MatIndex].Color;
		glColor4ubv(meshcol);
		

		if (g.FaceType == 1)
			glBegin(GL_TRIANGLE_STRIP);
		else
			glBegin(GL_TRIANGLES);

		for (k = 0; k < s->numIndices; k++) {
			i = s->Indices[k];
			if ((g.Flags & rpGEOMETRYTEXTURED) ||
			     (g.Flags & rpGEOMETRYTEXTURED2))
				glMultiTexCoord2fv(GL_TEXTURE0, uv[i]);
			if (g.Flags & rpGEOMETRYPRELIT && UseVertexColors) {
				if (mixColors) {
					MixColors(avcolor, color1[i], color2[i],
						  60-Clock.GetMinute(), 60);
				} else {
					avcolor[0] = color1[i][0];
					avcolor[1] = color1[i][1];
					avcolor[2] = color1[i][2];
					avcolor[3] = color1[i][3];
				}
				avcolor[0]=(meshcol[0]+avcolor[0])/2;
				avcolor[1]=(meshcol[1]+avcolor[1])/2;
				avcolor[2]=(meshcol[2]+avcolor[2])/2;
				avcolor[3]=(meshcol[3]+avcolor[3])/2;
				glColor4ubv(avcolor);
			}

			if (g.Flags & rpGEOMETRYNORMALS)
				glNormal3fv(norm[i]);
			glVertex3fv(vert[i]);
		}
		glEnd();
	}
}

void CWorldObj::Draw(bool DoTransformation, bool DrawTransp)
{
	RwInt32 g, f;

	if (ObjCount > Clump->numAtomics)
		return;
	g = Clump->Atm[ObjCount-1].GeoIndex;
	f = Clump->Atm[ObjCount-1].FrmIndex;

	glPushMatrix();
	if (DoTransformation)
		TransformFrame(Clump->Frm, f);
	DrawGeometry(Clump->Geo[g], DrawTransp);
	glPopMatrix();
}

void CWorldObj::TransformFrame(RwFrame *f, RwInt32 which)
{
	GLfloat transmat[16];

	if (f[which].Parent != -1)
		TransformFrame(f, f[which].Parent);

	transmat[0] = f[which].Rotation[0][0];
	transmat[1] = f[which].Rotation[0][1];
	transmat[2] = f[which].Rotation[0][2];
	transmat[3] = 0;

	transmat[4] = f[which].Rotation[1][0];
	transmat[5] = f[which].Rotation[1][1];
	transmat[6] = f[which].Rotation[1][2];
	transmat[7] = 0;

	transmat[8] = f[which].Rotation[2][0];
	transmat[9] = f[which].Rotation[2][1];
	transmat[10] = f[which].Rotation[2][2];
	transmat[11] = 0;

	transmat[12] = f[which].Position[0];
	transmat[13] = f[which].Position[1];
	transmat[14] = f[which].Position[2];
	transmat[15] = 1;

	glMultMatrixf(transmat);
}

void CWorldObj::SetClump(RpClump *c)
{
	Clump = c;
}

RpClump *CWorldObj::GetClump()
{
	return Clump;
}
